﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CleanUsingXmlns
{
    class Program
    {
        static void Main(string[] args)
        {
            const string usage = "usage: CleanUsingXmlns {addusing|removeusing} {path} {namespace1,namespace2} {externalAssembly1,externalAssembly2}";

            bool inputOk = true;
            bool removeUsing = true;
            bool madeAChange = false;
            int fileChangeCount = 0;

            if (args.Count() < 3)
            {
                inputOk = false;
                Console.WriteLine(usage);
            }
            else
                if (args[0] != "addusing" && args[0] != "removeusing")
                {
                    Console.WriteLine(usage);
                    inputOk = false;
                }
                else
                    if ((false == System.IO.Directory.Exists(args[1])))
                    {
                        Console.WriteLine("CleanUsingXmlns - invalid directory: " + args[1]);
                        inputOk = false;
                    }



            if (inputOk)
            {
                string[] nameSpacesToReplace = args[2].Split(',');
                List<string> externalAssemblies = new List<string>();
                if (args.Length > 3)
                {
                    externalAssemblies.AddRange(args[3].Split(','));
                }

                if (args[0] == "addusing")
                {
                    removeUsing = false;
                }

                List<string> directories = new List<string>(System.IO.Directory.GetDirectories(args[1]));

                directories.Insert(0, args[1]);

                foreach (string directory in directories)
                {
                    if (directory.ToLower().EndsWith("\\obj")
                        || directory.ToLower().EndsWith("\\bin"))
                        continue;

                    foreach (string file in System.IO.Directory.GetFiles(directory))
                    {
                        if (file.EndsWith(".xaml"))
                        {

                            madeAChange = false;

                            List<string> sbNew = new List<string>();
                            string[] fileLines = File.ReadAllLines(file);
                            bool insideHeader = true;

                            foreach (string line in fileLines)
                            {
                                string newLine = line;
                                if (insideHeader)
                                {
                                    if (newLine.Contains('>'))
                                    {
                                        insideHeader = false;
                                    }

                                    if (newLine.Contains("xmlns:"))
                                    {
                                        foreach (string namesp in nameSpacesToReplace)
                                        {
                                            if (removeUsing)
                                            {
                                                // convert from METRO USING to SL namespace
                                                string nsToken = "=\"using:" + namesp;
                                                if (newLine.Contains(nsToken))
                                                {
                                                    int startPos = newLine.IndexOf(nsToken) + 1;
                                                    int endPos = newLine.IndexOf("\"", startPos + 1);
                                                    if (startPos >= 0 && endPos >= 0)
                                                    {
                                                        string quoted = newLine.Substring(startPos, endPos - startPos);
                                                        string nsPart;
                                                        nsPart = quoted.Substring(quoted.IndexOf(':') + 1);

                                                        // check for external assembly include'
                                                        bool includeExternalAssembly = false;
                                                        foreach (string extAssmbly in externalAssemblies)
                                                        {
                                                            if (nsPart.ToLower().Contains(extAssmbly))
                                                            {
                                                                includeExternalAssembly = true;
                                                                break;
                                                            }
                                                        }

                                                        if (includeExternalAssembly)
                                                            nsPart = "\"clr-namespace:" + nsPart +";assembly=" + nsPart;
                                                        else
                                                            nsPart = "\"clr-namespace:" + nsPart;

                                                        
                                                        newLine = newLine.Replace(quoted, nsPart);
                                                        madeAChange = true;
                                                    }
                                                }


                                            }
                                            else
                                            {
                                                // convert from SL namespace to METRO USING
                                                string nsToken = "=\"clr-namespace:" + namesp;
                                                if (newLine.Contains(nsToken))
                                                {
                                                    int startPos = newLine.IndexOf(nsToken) + 1;
                                                    int endPos = newLine.IndexOf("\"", startPos + 1);
                                                    if (startPos >= 0 && endPos >= 0)
                                                    {
                                                        string quoted = newLine.Substring(startPos, endPos - startPos);
                                                        string nsPart;
                                                        if (quoted.IndexOf('=') >= 0)
                                                            nsPart = "\"using:" + quoted.Substring(quoted.IndexOf('=') + 1);
                                                        else
                                                            nsPart = "\"using:" + quoted.Substring(quoted.IndexOf(':') + 1);

                                                        newLine = newLine.Replace(quoted, nsPart);
                                                        madeAChange = true;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }

                                sbNew.Add(newLine);
                            }

                            if (madeAChange)
                            {
                                File.WriteAllLines(file, sbNew.ToArray());
                                Console.WriteLine("CleanUsingXmlns updated file: {0}", file);
                                fileChangeCount++;
                            }

                        }
                    }
                    Console.WriteLine("****************************");
                    Console.WriteLine("*** CleanUsingXmlns updated {0} files total in {1}.", fileChangeCount, directory);
                    Console.WriteLine("****************************");
                    fileChangeCount = 0;
                }
            }

        }
    }
}
