﻿/////////////////////////////////////////////////////////////////
//
// Silverlight + FarSeer Physics Helper
//
// by Andy Beaulieu - http://www.andybeaulieu.com
//
// LICENSE: This code is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. ANY 
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS
// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
/////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
#if METRO
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Media;
#else
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Xna.Framework.Audio;
#endif
#if WINDOWS_PHONE
using Microsoft.Xna.Framework.Audio;
using System.IO;
using Microsoft.Xna.Framework;
#endif


namespace Spritehand.FarseerHelper
{
    public class SoundMain
    {
        public static bool SoundEnabled = true;

#if SILVERLIGHT
        private List<SoundEffectInstance> _sounds;
#else
        private List<SoundElement> _sounds;
#endif
        private double _loopTime = 0;
        private double _loopIndex = 0;
        private bool _loopPlaying;
        private string _name;
        private static int _soundId = 0;
        private int _currentSoundIndex = 0;

#if WINDOWS_PHONE
        SoundEffectInstance _effect;

        public SoundEffectInstance Effect
        {
            get
            {
                return _effect;
            }
        }
#endif
        public DateTime LastPlayTime { get; set; }


        public string Name
        {
            get
            {
                return _name;
            }
            set
            {
                _name = value;
            }
        }
        public double LoopTime
        {
            get
            {
                return _loopTime;
            }

            set
            {
                _loopTime = value;
            }
        }

        private double _volume;
        public double Volume
        {
            get
            {
                return _volume;
            }
            set
            {
                _volume = value;

#if WINDOWS_PHONE
                if (_effect != null)
                    _effect.Volume = (float)value;
#elif SILVERLIGHT
                foreach (SoundEffectInstance sound in _sounds)
                    sound.Volume = (float)value;
#else
                foreach (SoundElement sound in _sounds)
                    sound.Sound.Volume = value;
#endif
            }
        }

        private double _pitch;
        public double Pitch
        {
            get
            {
                return _pitch;
            }
            set
            {
                _pitch = value;
#if WINDOWS_PHONE
                if (_effect != null && value <= 1)
                    _effect.Pitch = (float)value;
#else
                throw new NotImplementedException("Pitch not supported");
#endif
            }
        }

        public SoundMain(Canvas container, string soundFile, int numBuffers, double loopTime)
        {

            LoopTime = loopTime;

            // create the sound elements


#if WINDOWS_PHONE
            // use XNA for sound - mixing is done for us
            if (soundFile.StartsWith("/"))
                soundFile = soundFile.Substring(1);
            Stream stream = TitleContainer.OpenStream(soundFile);
            SoundEffect effect = SoundEffect.FromStream(stream);
            _effect = effect.CreateInstance();
            if (loopTime > 0)
                _effect.IsLooped = true;
#elif SILVERLIGHT
            _sounds = new List<SoundEffectInstance>();
            for (int i = 0; i < numBuffers; i++)
            {
                _soundId++;
                // get rid of initial backslash "/"
                if (soundFile.StartsWith("/"))
                    soundFile = soundFile.Substring(1);
                var sndStream = Application.GetResourceStream(new Uri(soundFile, UriKind.RelativeOrAbsolute));
                SoundEffect sndEffect = SoundEffect.FromStream(sndStream.Stream);
                _sounds.Add(sndEffect.CreateInstance());
            }
#endif


#if METRO
            _sounds = new List<SoundElement>();
            
            Volume = -1;

           // use MediaElement for Sound - one for each "buffer"
            for (int i = 0; i < numBuffers; i++)
            {
                _soundId++;


               
                // create the sound from XAML
                SoundElement theSound = new SoundElement();
                theSound.Sound.AutoPlay = false;

                theSound.Sound.Source = new Uri(container.BaseUri, soundFile);
                string shortName = soundFile.Substring(soundFile.LastIndexOf("/") + 1, soundFile.LastIndexOf(".") - 1 - soundFile.LastIndexOf("/"));
                //theSound.Name = "sound" + shortName + _soundId.ToString();

                // add the sound to our List
                _sounds.Add(theSound);

                theSound.Sound.MarkerReached += new TimelineMarkerRoutedEventHandler(theSound_MarkerReached);
                theSound.Sound.MediaOpened += new RoutedEventHandler(theSound_MediaOpened);

                // add the sound to our main page
                container.Children.Add(theSound.Sound);
            }
#endif

        }


#if METRO
        void theSound_MarkerReached(object sender, TimelineMarkerRoutedEventArgs e)
        {

            MediaElement sound = (sender as MediaElement);
            if (LoopTime > 0 && _loopPlaying)
            {
                _loopIndex = (_loopIndex + 1) % _sounds.Count;
                for (int i = 0; i < _sounds.Count; i++)
                {
                    if (i == _loopIndex)
                    {
                        _sounds[i].Sound.Position = TimeSpan.FromMilliseconds(0);
                        _sounds[i].Sound.Volume = (Volume == -1) ? 0.5 : Volume;
                        _sounds[i].Sound.Play();
                    }
                }
            }
        }

        void theSound_MediaOpened(object sender, RoutedEventArgs e)
        {
            if (LoopTime > 0)
            {
                // set up our loop marker (this avoids gaps in loop)
                MediaElement sound = (sender as MediaElement);

                TimelineMarker marker = new TimelineMarker();
                marker.Text = "x";
                marker.Type = "y";
                marker.Time = TimeSpan.FromMilliseconds(LoopTime);

                sound.Markers.Add(marker);
            }
        }
#endif

        public void Play()
        {

            if (SoundMain.SoundEnabled == false)
                return;

            _loopPlaying = true;

#if WINDOWS_PHONE
            _effect.Volume = (float)((Volume == -1) ? 1 : Volume);
            _effect.Play();
            LastPlayTime = DateTime.Now;
#elif SILVERLIGHT
            _sounds[_currentSoundIndex].Play();
            _currentSoundIndex = (_currentSoundIndex + 1) % _sounds.Count;

            LastPlayTime = DateTime.Now;
#endif

#if (METRO)
            _sounds[_currentSoundIndex].Sound.Volume = (Volume == -1) ? 0.5 : Volume;
            if (_sounds[_currentSoundIndex].ReadyToPlay)
            {
                _sounds[_currentSoundIndex].Sound.Play();
                System.Diagnostics.Debug.WriteLine("played sound {0} with volume {1}", _sounds[_currentSoundIndex].Sound.Source.ToString(), _sounds[_currentSoundIndex].Sound.Volume);

                _currentSoundIndex = (_currentSoundIndex + 1) % _sounds.Count;

                LastPlayTime = DateTime.Now;


            }
            else
            {
                _sounds[_currentSoundIndex].PlayOnLoad = true;
            }
#endif


        }

        public void Stop()
        {
            _loopPlaying = false;
#if SILVERLIGHT
            // Stop() not supported on SoundEffect
            // TODO: zero out Volume?
#else
#if (!WINDOWS_PHONE)
            foreach (SoundElement sound in _sounds)
                sound.Sound.Stop();
#else
            _effect.Stop();
#endif
#endif
        }

        public void Cleanup()
        {
#if SILVERLIGHT
#else
            foreach (SoundElement sound in _sounds)
            {
                sound.Sound.MarkerReached -= theSound_MarkerReached;
                sound.Sound.MediaOpened -= theSound_MediaOpened;
                sound.CleanUp();
            }
#endif
        }

    }

    public class SoundElement
    {
        public bool ReadyToPlay { get; set; }
        public MediaElement Sound { get; set; }
        public bool PlayOnLoad { get; set; }

        public SoundElement()
        {
            Sound = new MediaElement();
            Sound.MediaOpened += new RoutedEventHandler(Sound_MediaOpened);
        }

        void Sound_MediaOpened(object sender, RoutedEventArgs e)
        {
            ReadyToPlay = true;

            if (PlayOnLoad && SoundMain.SoundEnabled)
                Sound.Play();
        }

        public void CleanUp()
        {
            Sound.MediaOpened -= Sound_MediaOpened;
        }

    }
}
